﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using System.Linq;

public class LineWave : MonoBehaviour {

	public float speed = 1f;
	public float amplitude = 1f;
	public float spacing = 10f;
	public float width = 1f;
	public float materialScale = 1f;

	public MeshFilter filter;

	protected List<float> intervals = new List<float>();
	protected List<Vector3> pts = new List<Vector3>();
	protected List<Color> colors = new List<Color>();

	protected float angle = 0f;

	public bool invertNormals = false;

  // Needed to prevent jaggies in spacing.
  float smoothDeltaTime = 0f;

  public virtual void AddPointToLine( float nextPointInterval ) {
  	float origInterval = nextPointInterval;
  	nextPointInterval = UpdateForInterval( nextPointInterval );

    angle += Time.deltaTime / nextPointInterval;
    if (angle > 1e3f) {
      // Occasionally constrain to prevent huge values and losing precision.
      angle %= 2 * Mathf.PI;
    }
		float nextAmplitude = Mathf.Sin( angle ) * amplitude;

		for( int index = 0; index < pts.Count; index++ ) {
			pts[index] = pts[index] + new Vector3( 5 * Time.deltaTime, 0f, 0f );
		}

		Vector3 nextPos = new Vector3( 0f, nextAmplitude, 0f );
		intervals.Insert( 0, origInterval );
		pts.Insert( 0, nextPos );
		colors.Insert( 0, GetColor.getColorFromHSV( 1f - origInterval ) );

		Vector2 screenPos = Camera.main.WorldToViewportPoint( pts[pts.Count - 1] );

		print("Screen pos x of last point in line = " + screenPos.x);
		if( screenPos.x > 1.5f || screenPos.x < 0f ) {
			pts.RemoveAt( pts.Count - 1 );
			colors.RemoveAt( pts.Count - 1 );
			intervals.RemoveAt( pts.Count - 1 );
		}

		RegenerateLine();
	}

 //  	protected virtual void AddPointToLine( float nextPointInterval ) {
	// 	float nextAmplitude = Mathf.Sin( Time.time * speed ) * amplitude;

	// 	for( int index = 0; index < pts.Count; index++ ) {
	// 		pts[index] = pts[index] + new Vector3( nextPointInterval, 0f, 0f );
	// 	}

	// 	Vector3 nextPos = new Vector3( 0f, nextAmplitude, 0f );
	// 	pts.Insert( 0, nextPos );
	// 	colors.Insert( 0, GetColor.getColorFromHSV( nextPointInterval ) );
		
	// 	Vector2 screenPos = Camera.main.WorldToViewportPoint( pts[pts.Count - 1] );
	// 	if( screenPos.x > 1f || screenPos.x < 0f ) {
	// 		pts.RemoveAt( pts.Count - 1 );
	// 	}

	// 	RegenerateLine();
	// }

	void RegenerateLine() {

		// calculate vertex positions

		Vector3[] verts = new Vector3[pts.Count * 2];
		for( int index = 1; index < pts.Count; index++ ) {
			Vector3 pt = pts[index];
			Vector3 lastPt = pts[index - 1];

			Vector3 dirFromLastPt = (pt - lastPt).normalized;
			Vector3 lastCross = Vector3.Cross( dirFromLastPt, Vector3.forward );

			if( index == 1 ) {
				// generate the first two vertices

				verts[0] = lastPt + lastCross * width;
				verts[1] = lastPt - lastCross * width;
			}

			if( index == pts.Count - 1 ) {
				// generate the last two vertices

				if( pt == pts[0] ) {
					Vector3 dirToNextPt = (pts[1] - pts[0]).normalized;
					Vector3 nextCross = Vector3.Cross( dirToNextPt, Vector3.forward );

					Vector3 crossAvg = ((lastCross + nextCross) / 2f).normalized;

					verts[index * 2] = pt + crossAvg * width;
					verts[index * 2 + 1] = pt - crossAvg * width;

					verts[0] = pt + crossAvg * width;
					verts[1] = pt - crossAvg * width;
				}
				else {
					verts[verts.Length - 2] = pt + lastCross * width;
					verts[verts.Length - 1] = pt - lastCross * width;
				}
			}
			else {
				// if not the first or the last point, create intermediate line points

				Vector3 nextStep = pts[index + 1];
				Vector3 dirToNextPt = (nextStep - pt).normalized;
				Vector3 nextCross = Vector3.Cross( dirToNextPt, Vector3.forward );

				Vector3 crossAvg = ((lastCross + nextCross) / 2f).normalized;

				verts[index * 2] = pt + crossAvg * width;
				verts[index * 2 + 1] = pt - crossAvg * width;
			}
		}

		// for( int index = 1; index < pts.Count; index++ ) {
		// 	Debug.DrawLine( pts[index], pts[index - 1], Color.red );
		// }

		// for( int index = 1; index < verts.Length; index++ ) {
		// 	Debug.DrawLine( verts[index], verts[index - 1], Color.blue );
		// }

		// build triangles
		int[] tris = new int[pts.Count * 6 - 6];
		for( int index = 0; index < pts.Count - 1; index++ ) {
			tris[index * 6] = index * 2 + 1;
			tris[index * 6 + 1] = index * 2 + 0;
			tris[index * 6 + 2] = index * 2 + 2;

			tris[index * 6 + 3] = index * 2 + 1;
			tris[index * 6 + 4] = index * 2 + 2;
			tris[index * 6 + 5] = index * 2 + 3;
		}

		// uvs and colors
		Vector2[] uvs = new Vector2[pts.Count * 2];
		Color[] meshColors = new Color[pts.Count * 2];
		float uvy = 0f;
		for( int index = 0; index < pts.Count; index++ ) {
			uvs[index * 2 + 0] = new Vector2( 0f, uvy );
			uvs[index * 2 + 1] = new Vector2( 1f, uvy );

			meshColors[index * 2 + 0] = colors[index];
			meshColors[index * 2 + 1] = colors[index];

			uvy += materialScale;
		}

		// normals
		Vector3[] normals = new Vector3[verts.Length];
		Vector3 normal = -Vector3.forward;
		if( invertNormals ) normal = -normal;
		for( int index = 0; index < verts.Length; index++ ) {
			normals[index] = normal;
		}

		Mesh newMesh = new Mesh();
		newMesh.vertices = verts;
		newMesh.triangles = tris;
		newMesh.uv = uvs;
		newMesh.colors = meshColors;
		newMesh.normals = normals;
		newMesh.name = "Line";

		filter.mesh = newMesh;
	}

  public float UpdateForInterval(float interval) {
    if (smoothDeltaTime == 0f) {
      smoothDeltaTime = Time.deltaTime;
    } else {
      smoothDeltaTime = Mathf.Lerp(smoothDeltaTime, Time.deltaTime, 0.1f);
    }
    // Add to prevent infinite frequency.
    return interval * smoothDeltaTime * spacing + 0.01f;
  }
}
